@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import
 import os
 import subprocess
 
@@ -30,6 +29,6 @@ class SSHShellTests(TestCase):
 
         # verify there is not a passphrase set on key
         ret = subprocess.check_output(
-            ["ssh-keygen", "-f", self.priv_key, "-y"]
+            ["ssh-keygen", "-f", self.priv_key, "-y"], timeout=30,
         )
         self.assertTrue(ret.decode().startswith("ssh-rsa"))
